var ULBA = ULBA || {};

/**
 * Main application's singleton object. It creates and loads page, manages state of URL and browser's history records,
 * manages localCache.
 * 
 * @constructor
 * @requires ULBA.page
 * @public ULBA.pageController.loadPage
 * extraData's required fields:
 * extraData.result {XML} Server's response
 * extraData.request {String} URL of AJAX request
 *
 */

ULBA.pageController = new (function(){
	var self = this, pageNames = ['port-details', 'login', 'search', 'search-result', 'country-details', 'port-map-plans', 'support'], hash;
    this.currentPage = null;
    this.params = null;
    this.pages = {};
    this.location = '';

    this.$eventTarget = $('<div></div>');

    /**
     * @param {String} name Name of page
     * @param {Object} serverResponse result of AJAX request which will be filled in page's content area
     * IE 7 has own implementation
     */
    this.loadPage = function(name, serverResponse){
        /**
         * if param extraData doesn't exist we trying get it from localStorage
         * if it exists we write down it to localStorage
         */

        if(serverResponse){
            self.pages[name].saveIntoStorage(name, JSON.stringify(serverResponse));
        }

        loadPage(name,serverResponse);
    };

    /**
     * Special IE 7 implementation.
     * This implementation doesn't get AJAX result from localStorage but inits new AJAX request
     * AJAX request's URL is kept in extraData.request
     */
    if(ie7){
        this.loadPage = function(name, extraData){
            var self = this;

            /**
             * Restore page's state from Page field cachedState
             * @see ULBA.page.saveState
             */
            if(self.pages[name].cachedState && !extraData){
                loadPage(name, extraData);
            }else if(!extraData){
                //Inits new AJAX request
                extraData = $.parseJSON(self.pages[name].getFromStorage(name));

                if(extraData && extraData.request){
                    var nameOfCallBackFunction = '',
                        counter = 0;

                    while(nameOfCallBackFunction == ''){
                        if(!ULBA.API['callback_' + counter]){
                            nameOfCallBackFunction = 'callback_' + counter;
                        }
                        counter += 1;
                    }

                    ULBA.API[nameOfCallBackFunction] = function(data){
                        extraData.result = data;
                        loadPage(name, extraData);
                    };

                    $.ajax({
                        url: extraData.request,
                        crossDomain: true,
                        jsonpCallback: 'ULBA.API.' + nameOfCallBackFunction,
                        dataType: 'jsonp',
                        success:ULBA.API[nameOfCallBackFunction],
                        error : function(XMLHttpRequest, textStatus, errorThrown){
                            $.showAlert('<p><strong>' + XMLHttpRequest.statusText +  '</strong></p><p>' + XMLHttpRequest.responseText + '</p>', 3000);
                        }
                    });

                }else{
                    loadPage(name, extraData);
                }
            }else{

                //Load page if extraData is ready
                loadPage(name,extraData);

                var lightData = {};

                for(var i in extraData){
                    if(i != 'result'){
                        lightData[i] = extraData[i]
                    }
                }

                self.pages[name].saveIntoStorage(name, JSON.stringify(lightData));
            }
        };
    }

    /**
     *
     * @private
     * @param {String} name Name of page
     * @param {Object} serverResponse result of AJAX request which will be filled in page's content area
     *
     */
     function loadPage(name, serverResponse){
        var target;

        //Call saveState for current page
        if(self.currentPage){
            self.currentPage.saveState();
        }

        $('BODY').removeAttr('class').addClass(name);

        self.params.page = name;

        // Used when page is opened in new tab
        target = self.params.target || null;
        delete self.params.target;

        putParamsIntoURL();
        self.currentPage = self.pages[name];

        self.location = location.pathname.replace(/^.+\//gi,'') + location.hash;

        //Put just loaded page into browser's history
        $.history.load('page=' + name);

        //Use if the page is loaded by parts (like port-details) to inform other components that all parts are loaded
        //Each page which is loaded by parts should trigger this event in the end (see port-details.js)
        self.$eventTarget.unbind('onLazyLoad');

        // It uses in port-details.js and port-map-plans.js
        // May be deleted
        if(target){
           try{delete serverResponse.target}catch(e){}
        }

        self.currentPage.initMe(serverResponse);

        if(target && $('A[data-target=' + target + ']').size()>0){
            $('A[data-target=' + target + ']').trigger('click');
        }else if(target){
            //Use if the page is loaded by parts (like port-details) to inform other components that all parts are loaded
            //Wait while all components will be loaded.
            self.$eventTarget.bind('onLazyLoad',function(){
                $('A[data-target=' + target + ']').trigger('click');
            })
        }
    }

    //Parses self.params and puts result into URL
    function putParamsIntoURL(){
        var HASH = $.param(self.params);
        location.hash = HASH;
    }


    function checkIfURLWasChanged(){
        if(location.hash != '#' + $.param(self.params)) return true;

        return false;
    }

    function processURL(){
        var URLparams = self.parseHASH(location.hash);

        if(URLparams.page && self.pages[URLparams.page]){
            self.loadPage(URLparams.page)
        }
    }

   /*
    * Starting function. Runs once, when on load/reload
    * Creates all pages that contain array pageNames
    * Takes page name from URL and loads appropriate page
    * Also checks if user is logged. If isn't redirects to Login page
    * @private
    */
    $(function(){
        var currentPageName;

        $.history.init(function(){});

        self.params = ULBA.pageController.parseHASH(location.hash);

        if(!self.params.page || Storage.get('apikey')==null || Storage.get('apikey')=="undefined"){
            self.params.page='login';
        }else{
            ULBA.API.KEY = Storage.get('apikey');
        }

        putParamsIntoURL();

        //Bind saveState to unload
        $(window).unload(function(){
            self.currentPage.saveState();
        });

        currentPageName = self.params.page;

        self.$eventTarget.one(currentPageName + ':loaded', function(){
            self.loadPage(currentPageName);
        });

        //Creating pages
        for(var i = 0; i < pageNames.length; i++){
            self.pages[pageNames[i]] = new ULBA.page(pageNames[i]);
        }

        /* Init URL observer */
        setInterval(function(){
            var URLWasChanged = checkIfURLWasChanged();

            if(URLWasChanged)processURL();
        },100);

    });

	return this;
})();

ULBA.page = function(fileName){
    var self = this;

    this.originalHTML = null; //Loaded HTML
    this.style = null; // Loaded CSS
    this.js = null; //Loaded JavaScript

    //See this.saveState
    this.cachedState = null;

    this.loaded = null;

    this.data = {};

    if(!fileName)return false;

    this.getFileName = function(){
        return fileName
    };

    this.restoreState = function(){
        //Each page has own implementation
    };

    //Saves page's state into object's field. It's valid while site won't be reloaded.
    //After reloading state will be lost and page will be restored from localStorage (IE8, FF)
    this.saveState = function(){
        self.cachedState = $('#content').find('> DIV').detach();
    };

    this.saveIntoStorage = function(name,value){
        Storage.put(fileName + '.' + name, value);
    };

    this.getFromStorage = function(name){
        var result = Storage.get(fileName + '.' + name);
        if(!result || result==null || result=="undefined")result=null;
        return result
    };


    /*
    * Function restores page from field self.cachedState
    * Or restores resources from localStorage
    *
    * @public
    *
     */
    this.initMe = function(serverResponse){
        if(!serverResponse){
            serverResponse = $.parseJSON(self.getFromStorage(fileName))
        }

        if(self.cachedState){
            self.restoreState(serverResponse);
        }else{
            $('#content').html(self.originalHTML);
            $('HEAD').append('<style type="text/css" >' + self.style + '</style>');
            eval(self.js); //init function 'initCurrentPage'
            if(initCurrentPage)initCurrentPage(serverResponse);
        }
    };

    function setStatus(){
        self.loaded = (self.originalHTML != null  && self.style != null  && self.js != null) ? true : false;

        if(self.loaded){
            setTimeout(function(){
                ULBA.pageController.$eventTarget.trigger(fileName + ':loaded');
            }, 50);
        }
    }
    
    (function loadResources() {
        if(!Storage.get(fileName + '.originalHTML') || Storage.get(fileName + '.originalHTML') == 'undefined'){
            $.get('pages/html/' + fileName + '.html',function(responseText){
                self.originalHTML = responseText;
                Storage.put(fileName + '.originalHTML', responseText);
                setStatus();
            });
        }else{
            self.originalHTML = Storage.get(fileName + '.originalHTML');
            setStatus();
        }

        if(!Storage.get(fileName + '.style') || Storage.get(fileName + '.style') == 'undefined'){
            $.get('pages/css/' + fileName + '.css',function(responseText){
                self.style = responseText;
                Storage.put(fileName + '.style', responseText);
                setStatus();
            })
        }else{
            self.style = Storage.get(fileName + '.style');
            setStatus();
        }

        if(!Storage.get(fileName + '.js') || Storage.get(fileName + '.js') == 'undefined'){
            $.get('pages/js/' + fileName + '.js?v=1',function(responseText){
                self.js = responseText;
                Storage.put(fileName + '.js', responseText);
                setStatus();
            });
        }else{
            self.js = Storage.get(fileName + '.js');
            setStatus();
        }
    })();


    return this;
};

/*
* @param {String} value
* @returns {Object}
*
 */

ULBA.pageController.parseHASH = function(value){
    if(!value)return {};
    value = value.replace(/^#/, '');

    var arrValue = value.split('&'), temp;
    var resultObj = {};

    for(var i = 0; i < arrValue.length; i++){
        temp = arrValue[i].split('=');
        if(temp[0] && temp[1])resultObj[temp[0]] = temp[1]
    }

    return resultObj
};
