function initCurrentPage(extraData){
    var smallMap, currentPage = ULBA.pageController.currentPage;

    currentPage.restoreState = function(extraData){
        $('#content').empty().append(currentPage.cachedState);

        if(extraData.result){
            //Reset page's state to initial and fill content
            $('#content').empty().append(currentPage.originalHTML);
            showContent(extraData)
        }
    };

    function showContent(summary){
        extraData = summary || extraData;

        var port = extraData.result,
            locode = port.locode.toLowerCase(),
            countryName = countries[locode],
            portName = port.port,
            portId = port.piuid,
            portCoordinates = new GLatLng(port.lat, port.lng);

        $('DIV.breadcrumbs').find('SPAN').text(portName);

        if(port.has_detail){
            $('A.detailedInfo').unbind('click').click(function(event){
                var $self = $(this);
                event.preventDefault();

                ULBA.API.getPortDetails(portId, function(data, url){
                    $self.parents('h2').hide();
                    $('DIV.detailsInfo').get(0).innerHTML = data;
                    $('DIV.detailsInfo').find(' > H1').remove();

                    $('DIV.detailsInfo').find('A[href^=#]').click(function(event){
                        event.preventDefault();
                        var catName = $(this).attr('href').replace(/.*#/i, ''), offsetTop;
                        var $targetLink = $('A[name="' + catName + '"]');
                        if($targetLink.size()>0){
                            offsetTop = $targetLink.get(0).offsetTop;
                            $('HTML').get(0).scrollTop = offsetTop;
                        }
                    });
                });
            }).attr('data-port-id', portId);
        }else{
            $('A.detailedInfo').remove();
        }

        $('A.showMap').click(function(event){
            event.preventDefault();
            var $showMap = $(this).clone(true);
            $(this).text('View Full Map').attr('title','Load Google Map display').addClass('nobg').unbind('click')
            .click(prepareContentForPortMapPlans);

            var $hideMap = $('<a class="hide-map" href="hide-map" title="Hide map">Hide map</a>').click(function(event){
                event.preventDefault();
                $('A.showMap').replaceWith($showMap);
                $('#smallMap').hide();
            });

            $('#smallMap').empty().show();

            var KML = JSON2KML(port);

            smallMap=new GMap2(document.getElementById("smallMap"));
            smallMap.setCenter(portCoordinates,10);
            var mgr = new MarkerManager(smallMap);
            mgr.clearMarkers();
            kmlLayer = new EGeoXml("kmlLayer", smallMap, KML, {addmarker:addMark});
            kmlLayer.parse();

            function addMark(marker,name,desc,imagefile,n) {

                var latLng = marker.getLatLng(), correctedMarker,
                    icon = new GIcon({image: imagefile,iconAnchor: new GPoint(16,16), iconSize: new GSize(32, 32), infoWindowAnchor: new GPoint(16,16)});

                correctedMarker = new GMarker(latLng, icon);

                mgr.addMarker(correctedMarker,0);
            }

            var zoomLevel = smallMap.getBoundsZoomLevel(kmlLayer.bounds);
            if(zoomLevel>10)zoomLevel=10;

            smallMap.setCenter(kmlLayer.bounds.getCenter(),zoomLevel);

            $('#smallMap').append($hideMap);
        });

        var $country = $('<a title="View country details" href="index.html#page=port-details">' + countryName + '</a>')
        .click(function(event){
            event.preventDefault();

            ULBA.API.getCountryInfo(locode, function(data,url){
                ULBA.pageController.loadPage('country-details', {result : data, locode : locode, request: url});
            });
        });

        $('DIV.details')
        .find('H1.title')
        .append('<span>' + portName + ', </span>')
        .append($country);

        if(port.wx_id){
            ULBA.API.getWeather(port.piuid, function(data){
                var $weatherIcon = $('<IMG />'),
                    $weatherLink = $('<a></a>'),
                    forecasts = data.forecast,
                    today = forecasts[0];

                $weatherLink.attr('title', today.day_weather).attr('href', '/weather');
                $weatherIcon.attr('src', 'images/wx/w' + today.day_weather_type + '.gif').attr('title', today.day_weather);

                $weatherLink.append($weatherIcon);

                $('DIV.details').find('H1.title').append($weatherLink);

                $weatherLink.click(function(event){
                    event.preventDefault();

                    var $forecast_page = $('<div class="forecast_page"></div>'),
                    $title = $('<h1>' + portName + ', ' + countryName + '</h1>'),
                    $subTitle = $('<h2>This is a five day weather forecast for ' + portName + ' in ' + countryName +'.</h2>'),
                    $forecast = $('<div class="forecast"></div>');

                    $forecast_page.append($title, $subTitle);

                    for(var i in forecasts){
                        var forecast = forecasts[i],
                            $day = $('<div class="day"></div>'),
                            $date = $('<div class="date"></div>'),
                            $image = $('<img />'),
                            $temperature = $('<div class="temperature"></div>'),
                            $min = $('<div class="min"></div>'),
                            $max = $('<div class="max"></div>'),
                            date = new Date(forecast.date),
                            $dayOfWeek = $('<div class="dayOfWeek"></div>');

                        if(i==0){
                            $dayOfWeek.html('Today');
                        }else{
                            $dayOfWeek.html(ULBA.common.daysOfWeek[date.getUTCDay()]);
                        }

                        $date.html(forecast.date);

                        $image.attr('title',forecast.day_weather)
                        .attr('alt', forecast.day_weather)
                        .attr('src', 'images/wx/w' + forecast.day_weather_type + '.gif');

                        $min.html(forecast.night_temp + '&#176' + 'C');
                        $max.html(forecast.day_temp + '&#176' + 'C');

                        $temperature.append($max, $min);
                        $day.append($dayOfWeek, $date, $image, $temperature);

                        if(i == forecasts.length-1){
                            $day.addClass('last-child')
                        }

                        $forecast.append($day);
                    }

                    $forecast_page.append($forecast);

                    $.message($('<div></div>').append($forecast_page).html());
                 });
            })
        }

        if(port.has_summary>0){
            ULBA.API.getPortSummary(portId, function(data){
                $('DIV.details').find('DIV.features').get(0).innerHTML = data;
            });
        }else{
            $('DIV.details').find('DIV.features').html('No summary')
        }


        ULBA.API.getPortACEReports(portId, function(data){
            if(data.length==0){
                $('DIV.ACEreports').html('No reports')
            }else{
                $(data).each(function(index, item){
                	$report_icon='<img src="images/report.png"/>';
                    var $a = $('<a data-ace_id="' + item.ace_id + '" data-href="' + item.link + '" title="' + item.heading + '" href="index.html#page=port-details&target=' + item.link + '">' + $report_icon + item.heading + '</a>')
                        .click(prepareContentForPortMapPlans);
                    
                    $a.attr('uId',getRandomInt(1,999));
                    $a.attr('data-target', $a.attr('data-href'));
                    $('DIV.ACEreports').append($a);
                })
            }

            ULBA.pageController.$eventTarget.trigger('onLazyLoad')
        });

        addPlansAndPhotos();
        showPortsCoordinates();

        function addPlansAndPhotos(){
            if(port.plans_count==0){
                $('DIV.plansAndPhotos').html('No plans');
            }else{
                ULBA.API.getPortPlans(portId, function(data){
                    $.each(data.plans, function(index, item){
                        //var link = item.link || item.http || '';
						var link = 'http://www.findaport.com/port/'+portId+'/plan/'+item.plan_id+'.gif'

                        if(link !=''){
                        	var $plan_type = item.plan_type_id;
                        	if ($plan_type=='10'||$plan_type=='20') {$plan_type_icon='<img src="images/camera_sm.png"/>';}
                        		else {$plan_type_icon='<img src="images/maps_sm.png"/>';}
                            var $a = $('<a data-href="' + link + '?key=' + Storage.get('apikey') + '" target="_blank" title="' + item.title + '">' + $plan_type_icon + ' ' + item.title +  '</a>');
                            //var $a = $('<a data-href="' + link + '?key=' + Storage.get('apikey') + '" traget="_blank" title="' + item.title + '">' + item.title + '</a>');

                            $a.attr('uId',getRandomInt(1,999)).click(prepareContentForPortMapPlans);

                            $a.attr('href', 'index.html#page=port-details&target=' +  link);
                            $a.attr('data-target', link);
                            $('DIV.plansAndPhotos').append($a);
                        }else{
                            var $span = $('<span>' + link + '(no image available)</span>');
                            $('DIV.plansAndPhotos').append($span);
                        }
                    });
                });
            }
        }

        function showPortsCoordinates(){
            var DMSLat = {
                d : Math.abs( parseInt(portCoordinates.lat()) ),
                m : Math.abs( parseInt((portCoordinates.lat() - parseInt(portCoordinates.lat()))*60 ) ),
                dir : (portCoordinates.lat()>0?'N':'S')
            };

            DMSLat.m = DMSLat.m.toString().length==1?'0'+DMSLat.m:DMSLat.m;

            var DMSLng = {
                d : Math.abs( parseInt(portCoordinates.lng()) ),
                m : Math.abs( parseInt(  (portCoordinates.lng() - parseInt(portCoordinates.lng()))*60 ) ),
                dir : (portCoordinates.lng()>0?'E':'W')
            };

            DMSLng.d = DMSLng.d.toString().length==1?'0' + DMSLng.d:DMSLng.d;
            DMSLng.d = DMSLng.d.toString().length==2?'0' + DMSLng.d:DMSLng.d;
            DMSLng.m = DMSLng.m.toString().length==1?'0' + DMSLng.m:DMSLng.m;

            $('#portCoord').html(DMSLat.d + '°' +
                    DMSLat.m + '\'' +
                    DMSLat.dir + ' ' +
                    DMSLng.d + '°' +
                    DMSLng.m + '\'' +
                    DMSLng.dir);
            }

        function prepareContentForPortMapPlans(event){
            event.preventDefault();

            var $html = $($('DIV.maps-plans').clone());
            $html.find('#smallMap').remove();

            ULBA.pageController.loadPage('port-map-plans', {result : port,
                                                            html : $html.html(),
                                                            target : $(this).attr('uId')
                                                            });
        }
    }

    $(function(){
        $('DIV.breadcrumbs').find('A[title="Search Result"]').click(function(event){
            ULBA.pageController.loadPage('search-result');
            return false;
        }).end()
        .find('A[title=Search]').click(function(event){
            ULBA.pageController.loadPage('search');
            return false;
        });
        
        showContent(extraData)
    });
}